# Filename: evaluationSurv.r
# Title: Evaluation measures based on the survival results
#
# Author: Christoph Bernau 
# Email: <bernau>
# Date of creation: 17.4.2012
#
# Brief description:
#   Evaluates resampling results of different survival methods.
#
# Further comments and notes:
#   Also needed for tuning purposes.
#
# Arguments:
#   -survresult: output-object of method 'learnsurvival'
#   -measure: character indicating the evaluation metric to be used
#
# Value:
#   evaloutput
###############################################################################

### signature = 'survaggr' survaggr is a new class containing a list of
### survoutput-objects and additional slots (e.g. X,y,genesel,nbgene) which
### could be generated at the end of the new 'classification function'
setMethod("evaluate", signature(object = "survaggr"), function(object, measure = "CvPLogL", 
    newdata = NULL, newy = NULL, index = NULL, imputezero = FALSE, ...) {
    
    measurec <- new(measure)
    
    ### prepare list for additional results
    survresult <- object
    additional <- list()
    ll <- list(...)
    # some administration get full X and y from survaggr-object survresult
    
    if (is.null(newdata)) {
        Xc <- survresult@X  #getX(survresult)
        
        if (is.null(newy) != TRUE) 
            warning("Argument \"newy\" is ignored.")
        if (is.null(index) != TRUE) 
            warning("Argument \"index\" is ignored.")
        
        if (ncol(Xc) == 1 & sum(abs(Xc)) == 0) {
            Xc <- ll$X
            ll$X <- NULL
        }
        
        yc <- survresult@y  #getY(survresult)
        # get genesel from survaggr
        genesel <- survresult@genesel  #getGenesel(survresult)
        # get nbgene from survaggr
        nbgene <- survresult@nbgene  #getNbgene(survresult)
        
        # get threshold from survaggr
        threshold <- survresult@threshold  #getNbgene(survresult)
        add <- list(...)
        foldmeasures <- list()
        
        for (i in 1:length(survresult@survoutputlist)) {
            ### learnind
            learnind <- survresult@survoutputlist[[i]]@learnind
            ### list for calling predict
            ll2 <- ll
            ll2$object <- survresult@survoutputlist[[i]]
            ll2$type <- measurec@measuretype
            
            ### include geneselection results
            if (length(genesel@method) != 0) {
                Xi <- Xc[, genesel@rankings[[1]][i, 1:nbgene]]
                
                if (threshold != -1) {
                  impi <- genesel@importance[[1]][i, ]
                  
                  if (genesel@criterion == "coefficient") 
                    impi <- impi[impi > threshold] else impi <- impi[impi < threshold]
                  nbgene2 <- min(length(impi), nbgene)
                  Xi <- Xc[, genesel@rankings[[1]][i, 1:nbgene2]]
                  
                }
            } else {
                Xi <- Xc
            }
            
            
            
            
            # newdata
            ll2$newdata <- Xi[-learnind, ]
            
            # predict (testdata)
            predictionobject <- do.call(predict, args = ll2)
            
            # further arguments for evaluate
            newy <- yc[-learnind, ]
            ll$lptrain <- survresult@survoutputlist[[i]]@linear.predictor
            ll$ytrain <- yc[learnind, ]
            
            
            # evaluate
            foldmeasures[[i]] <- evaluate(predictionobject, newy, measurec, add = ll)
        }
        out <- new("evaloutput", result = foldmeasures, measure = measure, additional = additional)
    } else {
        if (length(survresult@survoutputlist) != 1 & is.null(index) == TRUE) 
            stop("Object contains several models and no index is provided")
        
        if (is.null(index) == TRUE) 
            index <- 1
        
        if (is.null(newy)) 
            stop("Model cannot be evaluated without survival response on test data. Please specify \"newy\".")
        
        Xtrain <- survresult@X[survresult@survoutputlist[[index]]@learnind, ]
        
        ## Fill in missing variables of newdata with zero if imputezero=TRUE
        if (imputezero) {
            n.missing.vars <- sum(!colnames(Xtrain) %in% colnames(newdata))
            if (n.missing.vars > 0) {
                zeros <- matrix(0, ncol = n.missing.vars, nrow = nrow(newdata))
                colnames(zeros) <- colnames(Xtrain)[!colnames(Xtrain) %in% colnames(newdata)]
                rownames(zeros) <- rownames(newdata)
                newdata <- cbind(newdata, zeros)
                rm(zeros)
            }
        }
        
        newdata <- newdata[, na.omit(match(colnames(Xtrain), colnames(newdata)))]
        
        if (!identical(colnames(newdata), colnames(Xtrain))) 
            stop("newdata is missing variables that existed in the training set.\nYou may either: 1) call evaluate() with imputezero=TRUE, or impute missing variables\nprior to calling evaluate().")
        
        
        
        # prepare arguments for predict () call
        ll <- ll2 <- list(...)
        ll$measure <- measure
        ll$object <- survresult@survoutputlist[[index]]
        ll$newy <- newy
        
        nbgene <- survresult@nbgene
        threshold <- survresult@threshold
        
        genesel <- survresult@genesel  #getGenesel(survresult)
        ### include geneselection results
        if (length(genesel@rankings) != 0) {
            newdata <- newdata[, genesel@rankings[[1]][index, 1:nbgene]]
            
            if (threshold != -1) {
                impi <- genesel@importance[[1]][index, ]
                
                if (genesel@criterion == "coefficient") 
                  impi <- impi[impi > threshold] else impi <- impi[impi < threshold]
                nbgene2 <- min(length(impi), nbgene)
                newdata <- newdata[, genesel@rankings[[1]][index, 1:nbgene2]]
            }
            
        }
        
        
        ll$newdata <- newdata
        out <- do.call("evaluate", args = ll)
    }
    
    return(out)
})

setMethod("evaluate", signature(object = "linpred"), function(object, newy, measure, 
    add = list()) {
    
    evaluate(measure, object@lp, newy, add)
})


### ---there's only a slight difference right now but I still think this
### differentiation of survprobs and linpred should be maintained
setMethod("evaluate", signature(object = "survprobs"), function(object, newy, measure, 
    add = list()) evaluate(measure, object@survprobs, newy, add))

### signaure: survoutput
setMethod("evaluate", signature(object = "survoutput"), function(object, newdata, 
    newy, measure = "CvPLogL", add = list(), ...) {
    measurec <- new(measure)
    additional <- list()
    ### prepare arguments for predict call
    ll <- list(...)
    ll$object <- object
    ll$newdata <- newdata
    ll$type <- measurec@measuretype
    
    predictionobject <- do.call(predict, args = ll)
    # evaluate
    
    add <- c(add, list(...))
    add$ytrain <- object@y
    add$lptrain <- object@linear.predictor
    
    foldmeasures <- list(evaluate(predictionobject, newy, measurec, add))
    new("evaloutput", result = foldmeasures, measure = measure, additional = additional)
})

### signature:modelC
setMethod("evaluate", signature(object = "ModelC"), function(object, newdata, newy, 
    measure = "CvPLogL", add = list(), lptrain = NULL, ytrain = NULL, ...) {
    
    measurec <- new(measure)
    additional <- list()
    ### prepare arguments for predict call
    ll <- list(...)
    ll$object <- object
    ll$newdata <- newdata
    ll$type <- measurec@measuretype
    
    predictionobject <- do.call(predict, args = ll)
    # evaluate
    add <- c(add, list(...))
    add$lptrain <- lptrain
    add$ytrain <- ytrain
    
    foldmeasures <- list(evaluate(predictionobject, newy, measurec, add))
    
    new("evaloutput", result = foldmeasures, measure = measure, additional = additional)
})


setMethod("evaluate", signature(object = "coxph"), function(object, ...) {
    # wrap the coxph object into a survhdcustom object and then just use
    # evaluate()
    evaluate(new("survhdcustom", predfun = function(object, ...) new("linpred", lp = predict(object@mod, 
        ...)), mod = object, extraData = list()), ...)
})
